/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package picturestoryeditor;

import java.util.*;
import java.io.*;
import java.net.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.PatternSyntaxException;
import javax.swing.JOptionPane;

/**
 * INIファイル風読み書きクラス
 * @author Administrator
 */
public class IniFile {
    private TreeMap<String, TreeMap<String, String>> SectionList = new TreeMap<String, TreeMap<String, String>>();
    public String FileName = new String();
    public static final String ABSTRACT_CR = "%CR%";
    public static final String ABSTRACT_LF = "%LF%";
    public static final Character CR = 0x0D;
    public static final Character LF = 0x0A;
    
    public boolean LoadFile() {
        try {
            File Source = new File(FileName);
            if (!Source.exists()) return false;
            Source = null;
            
            FileInputStream fis = new FileInputStream(FileName);
            InputStreamReader ir = new InputStreamReader(fis , "UTF-8");
            BufferedReader br = new BufferedReader(ir);
            
            String msg;
            TreeMap<String, String> Section = null;
            SectionList = null;
            SectionList = new TreeMap<String, TreeMap<String, String>>();
            
            //1行ずつ読み込む
            while (br.ready()) {
                msg = br.readLine();
                //終端に達していないがnullになった場合はリトライする
                if (msg == null) continue;
               
                //空行は無視
                if (msg.isEmpty()) continue;

                //コメント行も無視
                if (msg.startsWith(";")) continue;

                int Index1 = msg.indexOf("[");
                int Index2 = msg.indexOf("]", Index1);

                //セクションではない場合
                if (((Index1 == -1) || (Index2 == -1)) && (Section != null)) {
                    Index1 = msg.indexOf("=");

                    //不正な行
                    if (Index1 == -1) continue;

                    Section.put(msg.substring(0, Index1).trim(), msg.substring(Index1 + 1).trim());
                }

                //セクションの場合
                else if(Index1 < Index2) {
                    msg = msg.substring(Index1 + 1, Index2);
                    Section = new TreeMap<String, String>();
                    SectionList.put(msg, Section);
                }
            }
            fis.close();
            ir.close();
            fis.close();
        } catch (Exception e) {
          return false;
        }

        return true;
    };

    public boolean SaveFile() {
        try {
            //ファイルが存在したら削除する
            File Dest = new File(FileName);
            if (Dest.exists()) Dest.delete();
            Dest = null;

            //書き込み用に開く
            FileOutputStream fos = new FileOutputStream(FileName);
            OutputStreamWriter ow = new OutputStreamWriter(fos, "UTF-8");
            BufferedWriter bw = new BufferedWriter(ow);

            String msg;
            TreeMap<String, String> Section = null;
            Iterator it = SectionList.entrySet().iterator();

            //セクションの数だけループ
            while (it.hasNext()) {
                //セクション名を書き込み
                Map.Entry entry = (Map.Entry)it.next();
                String Key = (String)entry.getKey();
                bw.write("[" + Key + "]");
                bw.newLine();

                Section = (TreeMap<String, String>) entry.getValue();
                Iterator it2 = Section.entrySet().iterator();

                //要素の数だけループ
                while (it2.hasNext()) {
                    entry = (Map.Entry)it2.next();
                    Key = (String)entry.getKey();
                    String Value = (String)entry.getValue();

                    bw.write(Key + " = " + Value);
                    bw.newLine();
                }

                bw.newLine();
            }
            bw.close();
            ow.close();
            fos.close();
        } catch (Exception e) {
          return false;
        }


        return true;
    };

    public String ReadData(String Section, String Key, String DefaultValue, boolean IsDecode) {
        TreeMap<String, String> Temp = SectionList.get(Section);
        if (Temp == null) return DefaultValue;

        String Value = Temp.get(Key);
        if (Value == null) return DefaultValue;
        if (IsDecode) {
            try {
                Value = Value.replaceAll(ABSTRACT_CR, CR.toString());
            }
            catch (PatternSyntaxException e) {
               
            }
            try {
                Value = Value.replaceAll(ABSTRACT_LF, LF.toString());
            }
            catch (PatternSyntaxException e) {

            }

            //Value = URLDecoder.decode(Value, "UTF-16");
        }
        return Value;

    }
    
    public long ReadData(String Section, String Key, long DefaultValue, boolean IsDecode) {
        return Long.parseLong(ReadData(Section, Key, ((Long)DefaultValue).toString(), IsDecode));
    }
   
    public int ReadData(String Section, String Key, int DefaultValue, boolean IsDecode) {
        return Integer.parseInt(ReadData(Section, Key, ((Integer)DefaultValue).toString(), IsDecode));
    }

    public short ReadData(String Section, String Key, short DefaultValue, boolean IsDecode) {
        return Short.parseShort(ReadData(Section, Key, ((Short)DefaultValue).toString(), IsDecode));
    }

    public byte ReadData(String Section, String Key, byte DefaultValue, boolean IsDecode) {
        return Byte.parseByte(ReadData(Section, Key, ((Byte)DefaultValue).toString(), IsDecode));
    }

    public float ReadData(String Section, String Key, float DefaultValue, boolean IsDecode) {
        return Float.parseFloat(ReadData(Section, Key, ((Float)DefaultValue).toString(), IsDecode));
    }

    public double ReadData(String Section, String Key, double DefaultValue, boolean IsDecode) {
        return Double.parseDouble(ReadData(Section, Key, ((Double)DefaultValue).toString(), IsDecode));
    }

    public boolean ReadData(String Section, String Key, boolean DefaultValue, boolean IsDecode) {
        return Boolean.parseBoolean(ReadData(Section, Key, Boolean.toString(DefaultValue), IsDecode));
    }

    public void WriteData(String Section, String Key, String Value, boolean IsEncode) {
        if (IsEncode && (Value != null) && (!Value.isEmpty())) {
            try {
                Value = Value.replaceAll(ABSTRACT_CR, CR.toString());
            }
            catch (PatternSyntaxException e) {

            }
            try {
                Value = Value.replaceAll(ABSTRACT_LF, LF.toString());
            }
            catch (PatternSyntaxException e) {

            }

            
            //Value = URLEncoder.encode(Value, "UTF-16");
        }

        //セクションが存在しない場合は作成する
        TreeMap<String, String> Temp = SectionList.get(Section);
        if (Temp == null) {
            Temp = new TreeMap<String, String>();
            SectionList.put(Section, Temp);
        }

        Temp.put(Key, Value);
    }
    
    public void WriteData(String Section, String Key, long Value, boolean IsEncode) {
        WriteData(Section, Key, Long.toString(Value), IsEncode);
    }
    
    public void WriteData(String Section, String Key, double Value, boolean IsEncode) {
        WriteData(Section, Key, Double.toString(Value), IsEncode);
    }

    public void WriteData(String Section, String Key, boolean Value, boolean IsEncode) {
        WriteData(Section, Key, Boolean.toString(Value), IsEncode);
    }
    
    public void DeleteData(String Section, String Key) {
        TreeMap<String, String> Temp = SectionList.get(Section); 
        if (Temp == null) return;
        if (Temp.get(Key) == null) return;
        Temp.put(Key, null);
        Temp.remove(Key);
    }
    
    public void DeleteSection(String Section) {
        if (SectionList.get(Section) == null) return;
        SectionList.put(Section, null);
        SectionList.remove(Section);
    }

    public void Clear() {
        SectionList.clear();
    }

    public int getSectionCount() {
        return SectionList.size();
    }
}
