/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package picturestoryeditor;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JOptionPane;
import picturestoryeditor.effect.Fade;
import picturestoryeditor.effect.Transition;
import picturestoryeditor.encode.AudioEncoder;

/**
 *
 * @author Administrator
 */
public class Project {
    public int project_FPS = 1;
    public AutoResizeType project_Resize = AutoResizeType.ASPECT_RATIO_HOLD;
    public Resolutions project_Resolution = Resolutions.Nico_Standard;
    public List<Scene> SceneList = new LinkedList<Scene>();

    //レンダリング時に使用
    private BufferedImage imageCacheBlend = null;
    private BufferedImage imageCacheCurrent = null;
    private BufferedImage imageCacheNext = null;
    private int currentSceneIndex = 0;
    private int frameCount, frameMax, totalFrameCount = 0;
    private int changeFrameCount, changeFrameMax;
    private double frameTime, remainderTime;
    private boolean changeFlag = false;
    private boolean initialized = false;
    private boolean newScene = false;

    private Transition transition;
    private Fade fade;

    public int getTotalFrameCount() {
        return totalFrameCount;
    }
    
    public BufferedImage getImageCacheBlend() {
        return imageCacheBlend;
    }

    public boolean isInitialized() {
        return initialized;
    }
    
    public BufferedImage getImageCacheCurrent() {
        return imageCacheCurrent;
    }

    public int getCurrentSceneIndex() {
        return currentSceneIndex;
    }
    
    public int calcTotalFrameCount() {
        double frameTime = 1.0 / project_FPS;
        int Result = 0;
        
        //全体のサイズを求める
        for (int i = 0; i < SceneList.size(); ++i) {
            Scene s = SceneList.get(i);
            int sceneFrame = Math.max((int)(s.base_Time / frameTime), 1);

            //シーン切替エフェクトが有効・最後のシーンではない
            if ((s.change_Effect != ChangeEffectType.NORMAL) && (i != SceneList.size() - 1)) {
                sceneFrame += (int)(s.change_Time / frameTime);
            }

            Result += sceneFrame;
        }
        return Result;
    }
    
    public void SoundEncode(String fileName) {
        AudioInputStream input1 = null;
        AudioInputStream input2 = null;
        FileOutputStream outStream = null;
        
        int outSampleRate = 44100;
        int outBitCount = 16;
        int outChannels = 2;
        int totalDataSize = 0;
        AudioFormat outFormat = new AudioFormat(outSampleRate, outBitCount, outChannels, true, true);

        byte[] byteBuffer1;
        int[] intBuffer1;
        int[] intBuffer2;

        try {
            
            double frameTime = 1.0 / project_FPS;
            double sceneTime = 0;
            int sceneSamples = 0;

            double startVolume = 1, endVolume = 1;

            Scene s;

            //ファイルを作成する
            outStream = new FileOutputStream(fileName);

            //全体のサイズを求める
            totalDataSize += (((int)(calcTotalFrameCount() * frameTime * outSampleRate)) * outChannels) * (outBitCount / 8);

            //ヘッダを書き込む
            outStream.write(AudioEncoder.CreateWaveHeader(outSampleRate, (short)outBitCount, (short)outChannels, totalDataSize));

            //合成はシーン単位で行う
            for (int i = 0; i < SceneList.size(); ++i) {
                s = SceneList.get(i);
                double tempTime = s.base_Time / frameTime;
                sceneTime = (s.base_Time - (tempTime - Math.floor(tempTime)));
                
                //シーン切替エフェクトが有効・最後のシーンではない
                if ((s.change_Effect != ChangeEffectType.NORMAL) && (i != SceneList.size() - 1)) {
                    tempTime = s.change_Time / frameTime;
                    sceneTime += (s.change_Time - (tempTime - Math.floor(tempTime)));
                }
                
                sceneSamples = (((int)(sceneTime * outSampleRate)) * outChannels);
                byteBuffer1 = new byte[sceneSamples * (outBitCount / 8)];
                intBuffer1 = new int[sceneSamples];
                intBuffer2 = new int[sceneSamples];
                
                switch (s.sound_Control) {
                    case PLAY:
                    case FADE_IN: {
                        if (input1 != null) {
                            input1.close();
                            input1 = null;
                        }

                        input1 = AudioSystem.getAudioInputStream(new File(s.sound_BGMFile));
                        endVolume = s.sound_Volume / 100.0;

                        if (s.sound_Control == SoundControlType.FADE_IN) {
                            startVolume = 0;
                        }
                        else {
                            startVolume = endVolume;
                        }
                        break;
                    }
                    
                    case STOP: {
                        if (input1 != null) {
                            input1.close();
                            input1 = null;
                        }
                        break;
                    }
                    
                    case VOLUME: {
                        startVolume = s.sound_Volume / 100.0;
                        endVolume = startVolume;
                        break;
                    }
                    case FADE_OUT: {
                        startVolume = endVolume;
                        endVolume = 0;
                        break;
                    }
                }
                
                //BGMが有効
                if (input1 != null) {
                    int totalReadLength = 0;
                    
                    while (true) {
                        
                        //バッファに読み込む
                        //int readLength = input1.read(byteBuffer1, totalReadLength, byteBuffer1.length - totalReadLength);
                        int readLength = AudioEncoder.ConvertAudioInputStream(input1, outFormat, intBuffer1, totalReadLength, intBuffer1.length - totalReadLength);
                        totalReadLength += readLength;
                        
                        //ストリームの終わり
                        if ((readLength < 0)) {
                            //閉じる
                            input1.close();
                            input1 = null;

                            //ループが有効なら開き直す
                            if (s.sound_LoopEnabled) input1 = AudioSystem.getAudioInputStream(new File(s.sound_BGMFile));
                            else break;
                        }
                        else if (totalReadLength >= intBuffer1.length){
                            break;
                        }
                    }

                    if (s.sound_Control == SoundControlType.FADE_OUT) {
                         input1.close();
                         input1 = null;
                    }

                    double vol = startVolume;
                    double incVol = (endVolume - startVolume) / (intBuffer1.length * 0.5);
                    for (int ii = 0; ii < intBuffer1.length; ii += 2) {
                        //左チャンネル
                        intBuffer1[ii] = (int)(intBuffer1[ii] * vol);
                        intBuffer1[ii + 1] = (int)(intBuffer1[ii + 1] * vol);
                        vol += incVol;
                    }

                    startVolume = endVolume;
                }
                
                //SEが有効
                if ((new File(s.sound_SEFile)).exists()) {
                    input2 = AudioSystem.getAudioInputStream(new File(s.sound_SEFile));
                    AudioEncoder.ConvertAudioInputStream(input2, outFormat, intBuffer2, 0, intBuffer2.length);
                    input2.close();
                    input2 = null;
                    
                }
                
                //BGMとSEの合成
                AudioEncoder.Mix(intBuffer1, intBuffer2, byteBuffer1);
                
                //ファイルへ出力
                outStream.write(byteBuffer1);
                totalDataSize += byteBuffer1.length;
            } //for

            outStream.flush();
        } catch (UnsupportedAudioFileException ex) {
            Logger.getLogger(Project.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Project.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                if (input1 != null) {
                    input1.close();
                }
                if (input2 != null) {
                    input2.close();
                }
                if (outStream != null) {
                    outStream.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(Project.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    public BufferedImage copyImage(BufferedImage src) {
        BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
        dest.setData(src.getData());
        return dest;
    }

    public boolean isNewScene() {
        return newScene;
    }
    
    public void RenderBegin(int index) {
        if (initialized) RenderEnd();
        
        imageCacheBlend = new BufferedImage(project_Resolution.getWidth(), project_Resolution.getHeight(), BufferedImage.TYPE_INT_RGB);
        frameTime = 1000.0 / project_FPS;
        remainderTime = 0;
        totalFrameCount = 0;
        initialized = true;
        
        if (SceneList.size() - 1 < index) {
            currentSceneIndex = 0;
            return;
        }
        currentSceneIndex = index;
        imageCacheCurrent = RenderScene(index);
        initScene(index);
        
    }

    private void initScene(int index) {
        
        if (index < SceneList.size()) initScene(SceneList.get(index));
    }

    private void initScene(Scene s) {
        frameCount = 0;
        frameMax = (int)((s.base_Time * 1000) / frameTime);
        if (frameMax <= 0) frameMax = 1;

        changeFrameCount = 0;
        changeFrameMax = (int)((s.change_Time * 1000) / frameTime);

        changeFlag = false;
        
    }
    
    public boolean RenderFrame() {
        if ((SceneList.size() <= currentSceneIndex) || (SceneList.size() <= 0)) return false;
        if (!initialized) RenderBegin(0);
        totalFrameCount++;

        Scene s = SceneList.get(currentSceneIndex);
        newScene = false;
        
        //切替時のアニメーションを表示する。
        if (changeFlag) {
            
            if (changeFrameCount < changeFrameMax) {
                
                    switch (s.change_Effect) {
                        case FADE: {
                            if (changeFrameCount == 0) {
                                fade = new Fade();
                                fade.RenderBegin(imageCacheCurrent, imageCacheNext, imageCacheBlend, changeFrameMax);
                                
                            }
                            fade.RenderFrame();
                            break;
                        }

                        case TRANSITION: {
                            if (changeFrameCount == 0) {
                                transition = new Transition();
                                try {
                                    transition.RenderBegin(imageCacheCurrent, imageCacheNext, imageCacheBlend, ImageIO.read(new File(s.change_RuleImageFile)), changeFrameMax);
                                } catch (IOException ex) {
                                    Logger.getLogger(Project.class.getName()).log(Level.SEVERE, null, ex);
                                    JOptionPane.showMessageDialog(null, "ルールファイルの読み込みに失敗しました。\n" + ex, "Error", JOptionPane.ERROR_MESSAGE);
                                    return false;
                                }
                            }
                            transition.RenderFrame(4);
                            break;
                        }
                    }

                //最後のフレーム
                if (changeFrameCount == changeFrameMax - 1) {
                    changeFlag = false;
                    currentSceneIndex++;
                    imageCacheCurrent = imageCacheNext;
                    initScene(currentSceneIndex);
                }
                else {
                    changeFrameCount++;
                }
            }
            
        }
        
        else {
            //指定の時間分、同じフレームを表示し続ける
            if (frameCount < frameMax) {
                //最初のフレーム
                if (frameCount == 0) {
                    imageCacheBlend = copyImage(imageCacheCurrent);
                    newScene = true;
                }
                
                //最後のフレーム
                if (frameCount == frameMax - 1) {

                    switch (s.change_Effect) {
                        //アニメーションなし
                        case NORMAL: {
                            currentSceneIndex++;
                            imageCacheCurrent = RenderScene(currentSceneIndex);
                            initScene(currentSceneIndex);
                            break;
                        }
                        default: {
                            //最後のシーンではない
                            if (currentSceneIndex < SceneList.size() - 1) {
                                if (changeFrameMax >= 1) {
                                    changeFlag = true;
                                    imageCacheNext = RenderScene(currentSceneIndex + 1);
                                }
                                else {
                                    currentSceneIndex++;
                                    imageCacheCurrent = RenderScene(currentSceneIndex);
                                    initScene(currentSceneIndex);
                                }
                                
                            }

                            //最後のシーン
                            else {
                                currentSceneIndex++;
                            }
                            break;
                        }
                    }
                    
                }
                else {
                    frameCount++;
                }
            }
        }
        return true;
    }
    
    public BufferedImage RenderScene(int index) {
        if (index > SceneList.size() - 1) return null;
        else return RenderScene(SceneList.get(index));
    }
    
    public BufferedImage RenderScene(Scene s) {
        try {
            BufferedImage baseImage = ImageIO.read(new File(s.base_ImageFile));
            BufferedImage outImage = new BufferedImage(project_Resolution.getWidth(), project_Resolution.getHeight(), BufferedImage.TYPE_INT_RGB);
            Graphics2D g2 = outImage.createGraphics();
            g2.setBackground(Color.BLACK);
            g2.clearRect(0, 0, outImage.getWidth(), outImage.getHeight());
            
            int x = 0, y = 0, width = 0, height = 0;
            
            //リサイズ方法によって処理を分ける
            switch (project_Resize) {
                
                //リサイズはしない
                case DISABLE: {
                    width = baseImage.getWidth();
                    height = baseImage.getHeight();
                    x = (int)((project_Resolution.getWidth() - width) * 0.5);
                    y = (int)((project_Resolution.getHeight() - height) * 0.5);

                    break;
                }
                
                //動画のサイズに合わせる
                case NORMAL: {
                    x = 0;
                    y = 0;
                    width = project_Resolution.getWidth();
                    height = project_Resolution.getHeight();

                    break;
                }
                
                //アスペクト比維持
                case ASPECT_RATIO_HOLD: {
                    
                    //縦を基準にして倍率を決める
                    height = project_Resolution.getHeight();
                    width = (int)(baseImage.getWidth() * ((double)height / (double)baseImage.getHeight()));
                    
                    //横がはみ出す場合は横を基準にする
                    if (width > project_Resolution.getWidth()) {
                        width = project_Resolution.getWidth();
                        height = (int)(baseImage.getHeight() * ((double)width / (double)baseImage.getWidth()));
                    }
                    
                    x = (int)((project_Resolution.getWidth() - width) * 0.5);
                    y = (int)((project_Resolution.getHeight() - height) * 0.5);

                    break;
                }
            }

            g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2.drawImage(baseImage, x, y, width, height, null);
            g2.dispose();
            return outImage;
        } catch (IOException ex) {
            Logger.getLogger(Project.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }
    
    public void RenderEnd () {
        imageCacheCurrent = null;
        imageCacheBlend = null;
        imageCacheNext = null;
        
        initialized = false;
    }
    
    public void SaveToINI(String FileName) {
        IniFile ini = new IniFile(); 
        String sectionName = "Project";
        String keyName = "";
        
        ini.FileName = FileName;
        ini.WriteData(sectionName, keyName + "project_Resize", project_Resize.name(), false);
        ini.WriteData(sectionName, keyName + "project_FPS", project_FPS, false);
        ini.WriteData(sectionName, keyName + "project_Resolution", project_Resolution.name(), false);
        
        ini.WriteData(sectionName, keyName + "project_SceneCount", SceneList.size(), false);
        
        for (int i = 0; i < SceneList.size(); ++i) {
            SceneList.get(i).SaveToINI(ini, Integer.toString(i), "");
        }
        ini.SaveFile();
        
    }
    
    public void LoadFromINI(String FileName) {
        IniFile ini = new IniFile(); 
        String sectionName = "Project";
        String keyName = "";
        
        ini.FileName = FileName;
        ini.LoadFile();
        
        project_Resize = AutoResizeType.valueOf(ini.ReadData(sectionName, keyName + "project_Resize", AutoResizeType.ASPECT_RATIO_HOLD.name(), false));
        project_FPS = ini.ReadData(sectionName, keyName + "project_FPS", 1, false);
        project_Resolution = Resolutions.valueOf(ini.ReadData(sectionName, keyName + "project_Resolution", Resolutions.Nico_Standard.name(), false));
        
        int m = ini.ReadData(sectionName, keyName + "project_SceneCount", 0, false);
        
        for (int i = 0; i < m; ++i) {
            Scene s = new Scene();
            s.LoadFromINI(ini, Integer.toString(i), "");
            SceneList.add(s);
        }
        
    }

}
