/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package picturestoryeditor.effect;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBufferInt;

/**
 * トランジションを行うクラス
 * @author Administrator
 */
public class Transition {
    
    private BufferedImage ruleImage;
    private BufferedImage beforeImage;
    private BufferedImage afterImage;
    private BufferedImage destImage;
    private int frameMax;
    private int frameCount;
    private double transValue;
    private double incTransValue;

    public void RenderBegin(BufferedImage beforeImage, BufferedImage afterImage, BufferedImage destImage, BufferedImage ruleImage, int frameMax) {
        if ((ruleImage.getWidth() != destImage.getWidth()) || (ruleImage.getHeight() != destImage.getHeight()) || (ruleImage.getType() != BufferedImage.TYPE_BYTE_GRAY)) {
            this.ruleImage = new BufferedImage(destImage.getWidth(), destImage.getHeight(), BufferedImage.TYPE_BYTE_GRAY);
            Graphics2D g2 = this.ruleImage.createGraphics();
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
            g2.drawImage(ruleImage, 0, 0, destImage.getWidth(), destImage.getHeight(), null);
            g2.dispose();
        }
        else {
            this.ruleImage = ruleImage;
        }
        
        this.beforeImage = beforeImage;
        this.afterImage = afterImage;
        this.destImage = destImage;
        this.frameMax = frameMax;
        this.frameCount = 0;
        this.transValue = 0;
        this.incTransValue = 255.0 / (frameMax + 1);
    }
    
    public void RenderEnd() {
        
    }
    
    public boolean RenderFrame(int vague) {
        if (frameCount < frameMax) {
            transValue += incTransValue;
            execute(beforeImage, afterImage, destImage, ruleImage, (int)transValue, vague);
            frameCount++;
            return true;
        }
        else return false;
    }
    /**
     * TransValue = 0 ～ 255<br>
     * 切り替え段階<br>
     *
     * VagueValue = 1 ～ 9<br>
     * しきい値<br>
    Dest = 出力先<br>
    Rule = 8bitルール画像<br>
    Before = 切り替え前の画像<br>
    After = 切り替え後の画像<br>
    */
    public static void execute(BufferedImage BeforeImage, BufferedImage AfterImage, BufferedImage DestImage, BufferedImage RuleImage, int TransValue, int VagueValue) {
        int[] beforeBuffer, afterBuffer, destBuffer;
        byte[] ruleBuffer;
        
        beforeBuffer = ((DataBufferInt)BeforeImage.getRaster().getDataBuffer()).getData();
        afterBuffer = ((DataBufferInt)AfterImage.getRaster().getDataBuffer()).getData();
        destBuffer = ((DataBufferInt)DestImage.getRaster().getDataBuffer()).getData();
        ruleBuffer = ((DataBufferByte)RuleImage.getRaster().getDataBuffer()).getData();
        
        int height = BeforeImage.getHeight();
        int width = BeforeImage.getWidth();
        int pos = 0;

        VagueValue = 9 - VagueValue;
        
        for (int y = 0; y < height; ++y) {
            for (int x = 0; x < width; ++x) {
                
                /* ルール画像読み込み
                 * 0になったらAfterImageに切り替わる。
                 * つまりルール画像は、BeforeImageのアルファ係数を求めるものである。
                 */
                int beforeAlpha = (int)ruleBuffer[pos] & 0xFF;
                
                //TransValueを引く
                //切り替えが進むごとに0に近くなる。
                beforeAlpha -= TransValue;
                
                //0以下なら0にする
                if (beforeAlpha < 0) beforeAlpha = 0;

                //しきい値が256になるように調整
                beforeAlpha <<= VagueValue;

                //0-255にして1を足す。
                if (beforeAlpha > 255) beforeAlpha = 255;
                beforeAlpha++;
                
                //AfterImageのアルファ係数を求める
                int afterAlpha = 256 - beforeAlpha;
                
                /*
                 * ピクセルデータの計算
                 */
                int beforePixel = 0;
                int afterPixel = 0;
                
                //α
                beforePixel  = ((((beforeBuffer[pos] & 0xFF000000) >>> 24) * beforeAlpha) >>> 8) << 24;
                
                //R
                beforePixel |= ((((beforeBuffer[pos] & 0x00FF0000) >>> 16) * beforeAlpha) >>> 8) << 16;

                //G
                beforePixel |= ((((beforeBuffer[pos] & 0x0000FF00) >>> 8 ) * beforeAlpha) >>> 8) << 8;
                
                //B
                beforePixel |= ((((beforeBuffer[pos] & 0x000000FF)       ) * beforeAlpha) >>> 8);


                //α
                afterPixel  = ((((afterBuffer[pos] & 0xFF000000) >>> 24) * afterAlpha) >>> 8) << 24;
                
                //R
                afterPixel |= ((((afterBuffer[pos] & 0x00FF0000) >>> 16) * afterAlpha) >>> 8) << 16;

                //G
                afterPixel |= ((((afterBuffer[pos] & 0x0000FF00) >>> 8 ) * afterAlpha) >>> 8) << 8;
                
                //B
                afterPixel |= ((((afterBuffer[pos] & 0x000000FF)       ) * afterAlpha) >>> 8);
                
                //飽和加算
                int tmp = ((afterPixel & beforePixel) + (((afterPixel ^ beforePixel) >> 1) & 0x7f7f7f7f)) & 0x80808080;
                int mask = (tmp << 1) - (tmp >> 7);
                destBuffer[pos] = ((afterPixel + beforePixel) - mask) | mask;
                //destBuffer[pos] = afterPixel + beforePixel;

                //次のピクセルへ
                pos++;
            }
            
        }
    }

}
