/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package picturestoryeditor.encode;

import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;

/**
 * オーディオを合成する
 * @author Administrator
 */
public class AudioEncoder {
    public static int WAVE_HEADER_SIZE = 44;

    public static int ConvertAudioInputStream(AudioInputStream stream, AudioFormat format, int[] buffer, int offset, int length) throws IOException {
        AudioFormat inFormat = stream.getFormat();
        int inFrameSize = inFormat.getFrameSize();

        //出力１サンプル当たりの入力サンプル数
        double d_ib = 0, d_inc_ib = inFormat.getSampleRate() / format.getSampleRate();
        
        byte[] temp = new byte[(int)((inFormat.getSampleSizeInBits() / 8) * inFormat.getChannels() * (length / format.getChannels()) * d_inc_ib)];
        int readLength = stream.read(temp);
        if (readLength < 0) return readLength;

        int ib = 0;
        
        
        int sample = 0;
        for (int i = offset; i < offset + length; ++i) {

            //ビット数・エンディアンを変換
            switch (inFormat.getSampleSizeInBits()) {
                case 8: {
                    sample = (temp[ib]) & 0xFF;
                    switch (format.getSampleSizeInBits()) {
                        case 16: {
                            sample <<= 8;
                            sample -= Short.MAX_VALUE;
                            break;
                        }
                    }
                    break;    
                }
                
                case 16: {
                    if (inFormat.isBigEndian()) sample = (short)(((temp[ib] & 0xFF) << 8) | (temp[ib + 1] & 0xFF));
                    else sample = (short)((temp[ib] & 0xFF) | ((temp[ib + 1] & 0xFF) << 8));
                    break;
                }
            }
            buffer[i] = sample;

            //周波数・チャンネル数を変換
            if (((inFormat.getChannels() == 1) && (format.getChannels() == 2))) {
                //左右に同じサンプルを使ってステレオにする
                if ((i & 1) == 1) {
                    d_ib += d_inc_ib;
                    if (d_ib >= 1) {
                        ib += inFrameSize * (int)d_ib;
                        d_ib -= Math.floor(d_ib);
                    }
                }
            }
            else if (((inFormat.getChannels() == 2) && (format.getChannels() == 1))) {
                //左チャンネルだけを使ってモノラルにする
                d_ib += d_inc_ib;
                if (d_ib >= 1) {
                    ib += inFrameSize * (int)d_ib;
                    d_ib -= Math.floor(d_ib);
                }
            }
            else {
                d_ib += d_inc_ib;
                if (d_ib >= 1) {
                    ib += (inFormat.getSampleSizeInBits() / 8) * (int)d_ib;
                    d_ib -= Math.floor(d_ib);
                }
            }

            if (ib >= readLength) {
                return i + 1 - offset;
            }
        }
        return readLength;
    }

    public static byte[] CreateWaveHeader(int sampleRate, short sampleBit, short channel, int dataSize) {
        byte[] buffer = new byte[WAVE_HEADER_SIZE];
        
        buffer[0] = 0x52; //R
        buffer[1] = 0x49; //I
        buffer[2] = 0x46; //F
        buffer[3] = 0x46; //F
        
        //リトルエンディアンに変換する
        int temp = dataSize + WAVE_HEADER_SIZE - 8;
        buffer[4] = (byte)((temp & 0x000000FF));
        buffer[5] = (byte)((temp & 0x0000FF00) >>> 8);
        buffer[6] = (byte)((temp & 0x00FF0000) >>> 16);
        buffer[7] = (byte)((temp & 0xFF000000) >>> 24);
        
        buffer[8] = 0x57; //W
        buffer[9] = 0x41; //A
        buffer[10] = 0x56; //V
        buffer[11] = 0x45; //E
        
        buffer[12] = 0x66; //f
        buffer[13] = 0x6D; //m
        buffer[14] = 0x74; //t
        buffer[15] = 0x20; //半角スペース
        
        //fmtチャンクのサイズ
        buffer[16] = 0x10;
        buffer[17] = 0x00;
        buffer[18] = 0x00;
        buffer[19] = 0x00;
        
        //PCM
        buffer[20] = 0x01;
        buffer[21] = 0x00;
        
        buffer[22] = (byte)((channel & 0x00FF));
        buffer[23] = (byte)((channel & 0xFF00) >>> 8);
        
        buffer[24] = (byte)((sampleRate & 0x000000FF));
        buffer[25] = (byte)((sampleRate & 0x0000FF00) >>> 8);
        buffer[26] = (byte)((sampleRate & 0x00FF0000) >>> 16);
        buffer[27] = (byte)((sampleRate & 0xFF000000) >>> 24);
        
        //１秒あたりのデータ量
        temp = sampleRate * (sampleBit / 8) * channel;
        buffer[28] = (byte)((temp & 0x000000FF));
        buffer[29] = (byte)((temp & 0x0000FF00) >>> 8);
        buffer[30] = (byte)((temp & 0x00FF0000) >>> 16);
        buffer[31] = (byte)((temp & 0xFF000000) >>> 24);
        
        //１サンプルのサイズ×チャンネル数
        temp = (sampleBit / 8) * channel;
        buffer[32] = (byte)((temp & 0x000000FF));
        buffer[33] = (byte)((temp & 0x0000FF00) >>> 8);

        buffer[34] = (byte)((sampleBit & 0x000000FF));
        buffer[35] = (byte)((sampleBit & 0x0000FF00) >>> 8);
        
        buffer[36] = 0x64; //d
        buffer[37] = 0x61; //a
        buffer[38] = 0x74; //t
        buffer[39] = 0x61; //a
        
        buffer[40] = (byte)((dataSize & 0x000000FF));
        buffer[41] = (byte)((dataSize & 0x0000FF00) >>> 8);
        buffer[42] = (byte)((dataSize & 0x00FF0000) >>> 16);
        buffer[43] = (byte)((dataSize & 0xFF000000) >>> 24);
        
        return buffer;
    }
    
    /**
     * 16bit・2chのデータをミックスする
     * @param inputBuffer1
     * @param inputBuffer2
     * @param outputBuffer
     */
    public static void Mix(int[] inputBuffer1, int[] inputBuffer2, byte[] outputBuffer) {
        int sampleBlock = 1024;
        int index = 0;
        int maxValue;
        int ib;
        
        for (int i = 0; i < inputBuffer1.length / sampleBlock; ++i) {
            maxValue = 0;
            //2つのデータをミックスする
            for (int ii = index; ii < index + sampleBlock; ++ii) {
                inputBuffer1[ii] += inputBuffer2[ii];
                if (maxValue < Math.abs(inputBuffer1[ii])) maxValue = Math.abs(inputBuffer1[ii]);
            }
            
            //16bitの範囲を超えたら調整する
            if (maxValue > Short.MAX_VALUE - 1) {
                double ratio = (Short.MAX_VALUE - 1) / (double)maxValue;
                for (int ii = index; ii < index + sampleBlock; ++ii) {
                    inputBuffer1[ii] = (int)(inputBuffer1[ii] * ratio);
                }
            }
            
            ib = index * 2;
            
            //リトルエンディアンに変換
            for (int ii = index; ii < index + sampleBlock; ++ii) {
                outputBuffer[ib] = (byte)((short)inputBuffer1[ii] & 0x000000FF);
                outputBuffer[ib + 1] = (byte)((short)(inputBuffer1[ii] & 0x0000FF00) >>> 8);
                ib += 2;
            }
            
            index += sampleBlock;
        }

        /*
         * sampleBlock未満のデータを処理
         */
        
        maxValue = 0;
        //2つのデータをミックスする
        for (int ii = index; ii < inputBuffer1.length; ++ii) {
            inputBuffer1[ii] += inputBuffer2[ii];
            if (maxValue < Math.abs(inputBuffer1[ii])) maxValue = Math.abs(inputBuffer1[ii]);
        }

        //16bitの範囲を超えたら調整する
        if (maxValue > Short.MAX_VALUE - 1) {
            double ratio = (Short.MAX_VALUE - 1) / maxValue;
            for (int ii = index; ii < inputBuffer1.length; ++ii) {
                inputBuffer1[ii] = (int)(inputBuffer1[ii] * ratio);
            }
        }

        ib = index * 2;

        //リトルエンディアンに変換
        for (int ii = index; ii < inputBuffer1.length; ++ii) {
            outputBuffer[ib] = (byte)((short)inputBuffer1[ii] & 0x000000FF);
            outputBuffer[ib + 1] = (byte)((short)(inputBuffer1[ii] & 0x0000FF00) >>> 8);
            ib += 2;
        }
    }
}
