/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * JDialog_Encode.java
 *
 * Created on 2010/09/29, 11:25:53
 */

package picturestoryeditor.encode;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import picturestoryeditor.OSName;
import picturestoryeditor.PathUtil;
import picturestoryeditor.Project;
import picturestoryeditor.Resolutions;
import picturestoryeditor.SystemUtils;

/**
 *
 * @author Administrator
 */
public class JDialog_Encode extends javax.swing.JDialog implements Runnable {
    private static final String TOOLS_PATH = PathUtil.CURRENT_PATH + "tools" + File.separator;
    private static final String WINDOWS_X264_PATH = TOOLS_PATH + "x264.exe";
    private static final String WINDOWS_FAAC_PATH = TOOLS_PATH + "faac.exe";
    private static final String WINDOWS_NERO_PATH = TOOLS_PATH + "neroAacEnc.exe";
    private static final String WINDOWS_MP4BOX_PATH = TOOLS_PATH + "MP4Box.exe";

    private static final String WINDOWS_X64_X264_PATH = TOOLS_PATH + "x264_x64.exe";
    private static final String WINDOWS_X64_FAAC_PATH = TOOLS_PATH + "faac_x64.exe";
    private static final String WINDOWS_X64_NERO_PATH = TOOLS_PATH + "neroAacEnc_x64.exe";

    private static final String LINUX_X264_PATH = "x264";
    private static final String LINUX_FAAC_PATH = "faac";
    private static final String LINUX_NERO_PATH = "neroAacEnc";
    private static final String LINUX_MP4BOX_PATH = "MP4Box";

    //private static final String LINUX_X64_X264_PATH = "x264";
    //private static final String LINUX_X64_FAAC_PATH = "faac";
    //private static final String LINUX_X64_NERO_PATH = "neroAacEnc";

    
    //エコノミー回避は245だが、紙芝居なのでもっと低くてもいい
    private static final int AUDIO_BITRATE = 64;
    private static final int VIDEO_BITRATE_MAX = 120;
    private static final int VIDEO_BITRATE_MIN = 40;
    private static final int VIDEO_BITRATE_RANGE = VIDEO_BITRATE_MAX - VIDEO_BITRATE_MIN;
    
    //ビットレート計算の基準となる解像度
    private static final int BASE_PIXELS = Resolutions.Nico_Standard.getWidth() * Resolutions.Nico_Standard.getHeight();
    
    private int VIDEO_BITRATE = 120;
    private int TOTAL_BITRATE = AUDIO_BITRATE + VIDEO_BITRATE;

    //指定のビットレートで40MBに収まる時間(秒)
    private int BASE_TIME = 41943040 / ((TOTAL_BITRATE * 1000) / 8);

    private String fileName;
    private Project project;
    private boolean terminated = false;
    private Thread thread;
    
    //対応していないOSならfalse
    private boolean toolsEnabled;
    
    public boolean isTerminated() {
        return terminated;
    }

    public void setTerminated(boolean terminated) {
        this.terminated = terminated;
    }
    
    /** Creates new form JDialog_Encode */
    public JDialog_Encode(java.awt.Frame parent, boolean modal, String fileName, Project project) {
        super(parent, modal);
        initComponents();
        setLocationRelativeTo(this);

        switch (OSName.getOsName()) {
            case WINDOWS: 
            case LINUX:
            case SOLARIS: {
                toolsEnabled = true;
                break;
            }
            
            default: {
                toolsEnabled = false;
                break;
            }
        } 
        
        this.fileName = fileName;
        this.project = project;
        thread = new Thread(this);
        thread.start();
    }

    @Override
    public void run() {
        String tempFileName;
        
        double totalSecond = 0;
        double bps = 178;
        
        //規定の拡張子ではない場合は末尾に付加
        if (!fileName.toLowerCase(Locale.ENGLISH).endsWith(".mp4")) {
            fileName += ".mp4";
        }

        /*
         * 映像の中間ファイルを作成
         */
        
        tempFileName = PathUtil.AddLastSeparator(PathUtil.getParent(fileName)) + "temp" + Long.toString(System.currentTimeMillis()) + ".yuv";

        //レンダリングの準備
        project.RenderBegin(0);

        //中間ファイルを作成
        OutputStream os = null;
        try {
            os = new FileOutputStream(tempFileName);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        jRadioButton_VideoTemp.setSelected(true);

        while (project.RenderFrame()) {
            //処理の中止
            if (terminated) {
                project.RenderEnd();
                try {
                    os.flush();
                    os.close();
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }
                (new File(tempFileName)).delete();
                return;
            }
            
            try {
                os.write(YV12Converter.ConvertToYV12(project.getImageCacheBlend()));
            } catch (IOException ex) {
                Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        //動画全体の時間を取得
        totalSecond = project.getTotalFrameCount() / (double)project.project_FPS;
        
        project.RenderEnd();
        try {
            os.flush();
            os.close();
        } catch (IOException ex) {
            Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
        }

        /*
         * 映像をエンコード
         */

        //String VideoEncodeFileName = PathUtil.AddLastSeparator(PathUtil.getParent(fileName)) + "temp_video_" + Long.toString(System.currentTimeMillis()) + ".mp4";
        //CUIアプリで使用するコマンド
        List<String> commandList = new ArrayList<String>();
        
        if (toolsEnabled) {
            jRadioButton_Video_Encode.setSelected(true);

            //環境によってファイルを変える
            switch (OSName.getOsName()) {
                case WINDOWS: {
                    if (SystemUtils.OS_ARCH.equalsIgnoreCase("x64") && (new File(WINDOWS_X64_X264_PATH)).exists()) {
                        commandList.add(WINDOWS_X64_X264_PATH);
                    }
                    else {
                        commandList.add(WINDOWS_X264_PATH);
                    }
                    break;
                }
                case LINUX:
                case SOLARIS: {
                    commandList.add(LINUX_X264_PATH);
                    break;
                }
            }
            
            commandList.add("--pass");
            commandList.add("1");

            commandList.add("--input-csp");
            commandList.add("yv12");
            
            commandList.add("--input-res");
            commandList.add(Integer.toString(project.project_Resolution.getWidth()) + "x" + Integer.toString(project.project_Resolution.getHeight()));
            
            commandList.add("--fps");
            commandList.add(Integer.toString(project.project_FPS));
            
            commandList.add("--slow-firstpass");

            /*
             * 画質の設定
             */
            commandList.add("--preset");
            commandList.add("Medium");
            
            commandList.add("--profile");
            commandList.add("Main");
            
            commandList.add("--min-keyint");
            commandList.add(Integer.toString(project.project_FPS));
            
            commandList.add("--keyint");
            commandList.add(Integer.toString(project.project_FPS * 5));
            
            commandList.add("--b-adapt");
            commandList.add("2");

            commandList.add("--bframes");
            commandList.add("3");

            commandList.add("--weightb");

            commandList.add("--ref");
            commandList.add("3");
            
            commandList.add("--direct");
            commandList.add("auto");
            
            commandList.add("--me");
            commandList.add("hex");
            
            commandList.add("--no-fast-pskip");
            
            commandList.add("--no-dct-decimate");

            
            /*
             * 画質の設定ここまで
             */

            commandList.add("-B");

            /*
             * 40MBを超えないようにビットレートを調整する
             */

            //fpsによってビットレートを変動させる
            switch (project.project_FPS) {
                case 1: {
                    VIDEO_BITRATE = VIDEO_BITRATE_MIN;
                    break;
                }
                case 30: {
                    VIDEO_BITRATE = VIDEO_BITRATE_MAX;
                    break;
                }
                default: {
                    //fpsが低いほど急激に最大値に近づき、高くなるほど伸びが悪くなる
                    VIDEO_BITRATE = (int)((1 - Math.pow(1 - ((project.project_FPS - 1) / 29.0), 2)) * VIDEO_BITRATE_RANGE) + VIDEO_BITRATE_MIN;
                    break;
                }
            }
            
            //基準となる解像度とプロジェクトの解像度との比率でビットレートを変動させる
            VIDEO_BITRATE = (int)(VIDEO_BITRATE * ((project.project_Resolution.getWidth() * project.project_Resolution.getHeight()) / (double)BASE_PIXELS));
            
            TOTAL_BITRATE = AUDIO_BITRATE + VIDEO_BITRATE;
            BASE_TIME = 41943040 / ((TOTAL_BITRATE * 1000) / 8);

            //BASE_TIME秒以内ならVIDEO_BITRATE kbps
            if (totalSecond <= BASE_TIME) {
                bps = VIDEO_BITRATE;
            }
            else {
                bps = (TOTAL_BITRATE * (BASE_TIME / totalSecond)) - AUDIO_BITRATE;
            }
            commandList.add(Integer.toString((int)bps));

            commandList.add("-o");
            commandList.add(fileName);
            commandList.add(tempFileName);

            ProcessBuilder builder = new ProcessBuilder(commandList);
            builder.redirectErrorStream(true);

            Process process = null;

            //1pass
            try {
                try {
                    process = builder.start();
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }

                InputStream is = process.getInputStream();
                try {
                    //プロセスの終了待ち
                    while (is.read() >= 0) {
                        if (terminated) return;
                    }
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            finally {
                if(process != null) {
                    try {
                        process.getErrorStream().close();
                        process.getInputStream().close();
                        process.getOutputStream().close();
                        process.destroy();

                        if (terminated) {
                            (new File(tempFileName)).delete();
                            (new File(fileName)).delete();
                        }
                    } catch (IOException ex) {
                        Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }

            //2pass
            commandList.set(2, "2");
            builder = new ProcessBuilder(commandList);
            builder.redirectErrorStream(true);

            try {
                try {
                    process = builder.start();

                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }

                InputStream is = process.getInputStream();
                try {
                    //プロセスの終了待ち
                    while (is.read() >= 0) {
                        if (terminated) return;
                    }
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            finally {
                if(process != null) {
                    try {
                        process.getErrorStream().close();
                        process.getInputStream().close();
                        process.getOutputStream().close();
                        process.destroy();
                        (new File(tempFileName)).delete();
                        
                        //ログファイルの削除
                        File f = new File(PathUtil.AddLastSeparator(PathUtil.CURRENT_PATH) + "x264_2pass.log");
                        if (f.exists()) f.delete();
                        
                        f = new File(PathUtil.AddLastSeparator(PathUtil.CURRENT_PATH) + "x264_2pass.log.mbtree");
                        if (f.exists()) f.delete();
                        
                        
                        if (terminated) {
                            (new File(fileName)).delete();
                        }
                    } catch (IOException ex) {
                        Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
        }
        
        /*
         * 音声の中間ファイルを作成
         */

        jRadioButton_Audio_Temp.setSelected(true);
        tempFileName = PathUtil.AddLastSeparator(PathUtil.getParent(fileName)) + "temp" + Long.toString(System.currentTimeMillis()) + ".wav";
        project.SoundEncode(tempFileName);


        /*
         * 音声のエンコード
         */

        String AudioEncodeFileName = PathUtil.AddLastSeparator(PathUtil.getParent(fileName)) + "temp_audio_" + Long.toString(System.currentTimeMillis()) + ".mp4";
        if (toolsEnabled) {
            jRadioButton_Audio_Encode.setSelected(true);
            commandList.clear();
            
            boolean neroFlag = false;
            //Neroエンコーダがあればそれを優先する
            switch (OSName.getOsName()) {
                case WINDOWS: {
                    if (SystemUtils.OS_ARCH.equalsIgnoreCase("x64")) {
                        if ((new File(WINDOWS_X64_NERO_PATH)).exists()) {
                            commandList.add(WINDOWS_X64_NERO_PATH);
                            neroFlag = true;
                        }
                        else if ((new File(WINDOWS_NERO_PATH)).exists()){
                            commandList.add(WINDOWS_NERO_PATH);
                            neroFlag = true;
                        }
                        else if ((new File(WINDOWS_X64_FAAC_PATH)).exists()) {
                            commandList.add(WINDOWS_X64_FAAC_PATH);
                        }
                        else {
                            commandList.add(WINDOWS_FAAC_PATH);
                        }
                    }
                    else {
                        if ((new File(WINDOWS_NERO_PATH)).exists()){
                            commandList.add(WINDOWS_NERO_PATH);
                            neroFlag = true;
                        }
                        else {
                            commandList.add(WINDOWS_FAAC_PATH);
                        }
                    }
                    break;
                }
                
                case LINUX:
                case SOLARIS: {
                    if ((new File(LINUX_NERO_PATH)).exists()){
                        commandList.add(LINUX_NERO_PATH);
                        neroFlag = true;
                    }
                    else {
                        commandList.add(LINUX_FAAC_PATH);
                    }
                    break;
                }
            }
            
            if (neroFlag) {
                commandList.add("-2pass");
                commandList.add("-he");

                commandList.add("-br");
                commandList.add(Integer.toString(AUDIO_BITRATE * 1000));

                commandList.add("-if");
                commandList.add(tempFileName);
                commandList.add("-of");
                commandList.add(AudioEncodeFileName);
                
            }
            else {
                commandList.add("-b");
                commandList.add(Integer.toString(AUDIO_BITRATE));

                commandList.add("-o");
                commandList.add(AudioEncodeFileName);
                commandList.add(tempFileName);
                
            }
            
            ProcessBuilder builder = new ProcessBuilder(commandList);
            builder.redirectErrorStream(true);

            Process process = null;

            try {
                try {
                    process = builder.start();
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }

                InputStream is = process.getInputStream();
                try {
                    //プロセスの終了待ち
                    while (is.read() >= 0) {
                        if (terminated) return;
                    }
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);

                }
            }
            finally {
                if(process != null) {
                    try {
                        process.getErrorStream().close();
                        process.getInputStream().close();
                        process.getOutputStream().close();
                        process.destroy();
                        
                        //Waveファイルを削除
                        (new File(tempFileName)).delete();

                        if (terminated) {
                            //映像ファイルを削除
                            (new File(fileName)).delete();
                            
                            //音声ファイルを削除
                            (new File(AudioEncodeFileName)).delete();
                        }
                    } catch (IOException ex) {
                        Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
            
            /*
             * 映像と音声の合成
             */
            
            jRadioButton_Mix.setSelected(true);
            
            commandList.clear();
            switch (OSName.getOsName()) {
                case WINDOWS: {
                    commandList.add(WINDOWS_MP4BOX_PATH);
                    break;
                }
                case LINUX:
                case SOLARIS: {
                    commandList.add(LINUX_MP4BOX_PATH);
                    break;
                }
            }
            
            commandList.add("-add");
            commandList.add(fileName);
            
            commandList.add("-add");
            commandList.add(AudioEncodeFileName);
            
            commandList.add("-new");
            commandList.add(fileName);
            
            builder = new ProcessBuilder(commandList);
            builder.redirectErrorStream(true);

            process = null;

            try {
                try {
                    process = builder.start();
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }

                InputStream is = process.getInputStream();
                try {
                    //プロセスの終了待ち
                    while (is.read() >= 0) {
                        if (terminated) return;
                    }
                } catch (IOException ex) {
                    Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            finally {
                if(process != null) {
                    try {
                        process.getErrorStream().close();
                        process.getInputStream().close();
                        process.getOutputStream().close();
                        process.destroy();
                        
                        //音声ファイルを削除
                        (new File(AudioEncodeFileName)).delete();
                        
                        //映像ファイルを削除
                        //(new File(VideoEncodeFileName)).delete();

                    } catch (IOException ex) {
                        Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
            
        }

        
        /*
         * 完了
         */

        jRadioButton_Done.setSelected(true);

        JOptionPane.showMessageDialog(this, "エンコードが終了しました。");
        setVisible(false);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jRadioButton_VideoTemp = new javax.swing.JRadioButton();
        jRadioButton_Video_Encode = new javax.swing.JRadioButton();
        jRadioButton_Audio_Temp = new javax.swing.JRadioButton();
        jRadioButton_Audio_Encode = new javax.swing.JRadioButton();
        jRadioButton_Mix = new javax.swing.JRadioButton();
        jRadioButton_Done = new javax.swing.JRadioButton();
        jButton_Stop = new javax.swing.JButton();

        setTitle("エンコード");
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "エンコードの行程", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("MS UI Gothic", 0, 16), new java.awt.Color(0, 0, 0))); // NOI18N

        jRadioButton_VideoTemp.setFont(jRadioButton_VideoTemp.getFont().deriveFont((float)16));
        jRadioButton_VideoTemp.setText("映像の中間ファイルを作成");

        jRadioButton_Video_Encode.setFont(jRadioButton_Video_Encode.getFont().deriveFont((float)16));
        jRadioButton_Video_Encode.setText("映像をエンコード");

        jRadioButton_Audio_Temp.setFont(jRadioButton_Audio_Temp.getFont().deriveFont((float)16));
        jRadioButton_Audio_Temp.setText("音声の中間ファイルを作成");

        jRadioButton_Audio_Encode.setFont(jRadioButton_Audio_Encode.getFont().deriveFont((float)16));
        jRadioButton_Audio_Encode.setText("音声をエンコード");

        jRadioButton_Mix.setFont(jRadioButton_Mix.getFont().deriveFont((float)16));
        jRadioButton_Mix.setText("映像と音声を合成");

        jRadioButton_Done.setFont(jRadioButton_Done.getFont().deriveFont((float)16));
        jRadioButton_Done.setText("完了");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(31, 31, 31)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jRadioButton_Video_Encode)
                    .addComponent(jRadioButton_VideoTemp)
                    .addComponent(jRadioButton_Audio_Temp)
                    .addComponent(jRadioButton_Audio_Encode)
                    .addComponent(jRadioButton_Mix)
                    .addComponent(jRadioButton_Done))
                .addContainerGap(37, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jRadioButton_VideoTemp)
                .addGap(18, 18, 18)
                .addComponent(jRadioButton_Video_Encode)
                .addGap(18, 18, 18)
                .addComponent(jRadioButton_Audio_Temp)
                .addGap(18, 18, 18)
                .addComponent(jRadioButton_Audio_Encode)
                .addGap(18, 18, 18)
                .addComponent(jRadioButton_Mix)
                .addGap(18, 18, 18)
                .addComponent(jRadioButton_Done)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jButton_Stop.setFont(jButton_Stop.getFont().deriveFont((float)16));
        jButton_Stop.setText("中止");
        jButton_Stop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_StopActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(112, 112, 112)
                        .addComponent(jButton_Stop, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 20, Short.MAX_VALUE)
                .addComponent(jButton_Stop, javax.swing.GroupLayout.PREFERRED_SIZE, 37, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButton_StopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_StopActionPerformed
        terminated = true;
        try {
            if (thread != null) {
                thread.join();
                thread = null;
            }
        } catch (InterruptedException ex) {
            Logger.getLogger(JDialog_Encode.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        
    }//GEN-LAST:event_jButton_StopActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        jButton_StopActionPerformed(null);
    }//GEN-LAST:event_formWindowClosing


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton_Stop;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JRadioButton jRadioButton_Audio_Encode;
    private javax.swing.JRadioButton jRadioButton_Audio_Temp;
    private javax.swing.JRadioButton jRadioButton_Done;
    private javax.swing.JRadioButton jRadioButton_Mix;
    private javax.swing.JRadioButton jRadioButton_VideoTemp;
    private javax.swing.JRadioButton jRadioButton_Video_Encode;
    // End of variables declaration//GEN-END:variables

}
