/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package picturestoryeditor.encode;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;

/**
 * YV12を扱う変換クラス
 * @author Administrator
 */
public class YV12Converter {
    public static int getY(int r, int g, int b) {
        return ( (((263 * r) + (516 * g) + (100 * b)) >>> 10) + 16);
    }
    
    public static int getU(int r, int g, int b) {
        return ( ((-151 * r) - (298 * g) + (449 * b) + 0x20000) >>> 10);
    }

    public static int getV(int r, int g, int b) {
        return ( ((449 * r) - (377 * g) - (72 * b) + 0x20000) >>> 10);
    }
    
    /**
     * RGBをYV12に変換
     * @param image 変換元になる32bitイメージ。<br>
     * アルファは無視。<br>
     * 縦横共に2の倍数であること。
     * @return YV12に変換されたイメージのバイト配列
     */
    public static byte[] ConvertToYV12(BufferedImage image) {
        if (image == null) return null;
        int[] sourceBuffer = ((DataBufferInt)image.getRaster().getDataBuffer()).getData();
        int sourcePosition = 0;

        int width = image.getWidth();
        int height = image.getHeight();
        int planeSize = width * height;
        
        //変換元は24bit、変換後は12bit
        byte[] destBuffer = new byte[(int)(planeSize * 1.5)];

        int destYPosition = 0;
        int destVPosition = destYPosition + planeSize;
        int destUPosition = destVPosition + (planeSize / 4);
        
        int r1, r2, r3, r4;
        int g1, g2, g3, g4;
        int b1, b2, b3, b4;
        int rgb1, rgb2, rgb3, rgb4;
        
        //基本単位は2*2のブロック
        for (int y = 0; y < height >>> 1; ++y) {
            for (int x = 0; x < width >>> 1; ++x) {
                rgb1 = sourceBuffer[sourcePosition];
                rgb2 = sourceBuffer[sourcePosition + 1];
                rgb3 = sourceBuffer[sourcePosition + width];
                rgb4 = sourceBuffer[sourcePosition + width + 1];

                //赤を抽出
                r1 = (rgb1 & 0x00FF0000) >>> 16;
                r2 = (rgb2 & 0x00FF0000) >>> 16;
                r3 = (rgb3 & 0x00FF0000) >>> 16;
                r4 = (rgb4 & 0x00FF0000) >>> 16;

                //緑を抽出
                g1 = (rgb1 & 0x0000FF00) >>> 8;
                g2 = (rgb2 & 0x0000FF00) >>> 8;
                g3 = (rgb3 & 0x0000FF00) >>> 8;
                g4 = (rgb4 & 0x0000FF00) >>> 8;

                //青を抽出
                b1 = rgb1 & 0x000000FF;
                b2 = rgb2 & 0x000000FF;
                b3 = rgb3 & 0x000000FF;
                b4 = rgb4 & 0x000000FF;

                //輝度を出力
                destBuffer[destYPosition] = (byte) getY(r1, g1, b1);
                destBuffer[destYPosition + 1] = (byte) getY(r2, g2, b2);
                destBuffer[destYPosition + width] = (byte) getY(r3, g3, b3);
                destBuffer[destYPosition + width + 1] = (byte) getY(r4, g4, b4);
                
                //平均値を取る
                r1 = (r1 + r2 + r3 + r4) >>> 2;
                g1 = (g1 + g2 + g3 + g4) >>> 2;
                b1 = (b1 + b2 + b3 + b4) >>> 2;
                
                //色差を出力
                destBuffer[destVPosition] = (byte) getV(r1, g1, b1);
                destBuffer[destUPosition] = (byte) getU(r1, g1, b1);
                
                //位置を更新
                sourcePosition += 2;
                destYPosition += 2;
                destUPosition++;
                destVPosition++;
            }
            sourcePosition += width;
            destYPosition += width;
        }
        
        return destBuffer;
    }
}
